(****************************************************************************)
(* GetArg command-line option parser                                        *)
(* Copyright (C) 2012-2013 Quentin Heath                                    *)
(*                                                                          *)
(* This program is free software: you can redistribute it and/or modify     *)
(* it under the terms of the GNU General Public License as published by     *)
(* the Free Software Foundation, either version 3 of the License, or        *)
(* (at your option) any later version.                                      *)
(*                                                                          *)
(* This program is distributed in the hope that it will be useful,          *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of           *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *)
(* GNU General Public License for more details.                             *)
(*                                                                          *)
(* You should have received a copy of the GNU General Public License        *)
(* along with this program.  If not, see <http://www.gnu.org/licenses/>.    *)
(****************************************************************************)

(** Command line parser inspired by GNU getopt.
  *
  * This module offers an alternative to the standard Ocaml [Arg] module
  * with a support for short and long options (cf. [man 3 getopt] for an
  * example of C API).
  *
  * It is loosely inspired by the [Getopt] module written by Alain Frish,
  * but offers several improvements (hopefully):
  * - more complete support for long options with arguments ([--foo bar] is
  *   an alternative to [--foo=bar] if [--foo] must take an argument)
  * - more consistent support for short options with optional arguments
  *   ([-y 42] is equivalent to [--y-option 42], ie. an argument-less
  *   option and a plain argument, instead of being equivalent to [-y42]
  *   and thus to [--y-option=42], ie. an argument supplied to an option)
  * - more flexible support for concatenated short options (if [-z] takes
  *   no argument, [-zx 42] is legal even if [-x] may or must take an
  *   argument, and is equivalent to [-z -x 42] *)

(** {6 Options specification} *)

(** The options are specified by a list of
  * [((short,long,description),withoutarg,witharg)]:
  * - [short] is the short form of the option (['\\000'] for no short form)
  * - [long] is the long form of the option ([""] for no long form)
  * - [description] is the one-line description of the option.
  *   Like in [Arg], the first space splits the description into the displayed
  *   form of the argument (if needed) and the actual description
  *   ("<n> This option does…" becomes "--option <n>      This option does…").
  *   Unlike in [Arg], the descriptions are wrapped, and aligned by default
  * - [withoutarg] is an optional [unit -> unit] called for options without
  *   argument
  * - [witharg] is an optional function called for options with an argument
  *
  * Whether an option can or must take an argument is specified by
  * [(withoutarg,witharg)]:
  * - [None,NoArg]: silent option
  * - [None,_]: option with a mandatory argument
  * - [_,NoArg]: option with no argument
  * - [_,_]: option with an optional argument; the second function is called
  *   whenever an argument is found in collapsed form ([-xy42] or
  *   [--y-option=42] for [-y]/[--y-option]),
  *   otherwise the first is be used *)

(** Callback functions binding an option to its argument. *)
type opt_spec =
  | NoArg
    (** the option can't take an argument *)
  | String of (string -> unit)
    (** function called on the argument *)
  | Int of (int -> unit)
    (** function called on the converted argument *)
  | Float of (float -> unit)
    (** function called on the converted argument *)

exception Missing_argument of string
exception Unknown_option of string
exception Needless_argument of string
exception Incorrect_argument of string * string

(** {6 Options specification} *)

(** Display the usage message passed to {!parse}, then the list
  * of options. *)
val print_help : unit -> unit
(** For each option, the available forms are displayed, then an argument
  * if available, then a question mark if the argument is optional.
  * Last, the description (except for all that comes before the first space)
  * is displayed.
  * As a consequence, if an option takes no argument and the description
  * doesn't start with a space in the specification, its first word won't be
  * displayed. *)

(** [parse longopts process usage_msg] builds a help message to be displayed
  * with {!print_help}, then processes the command-line options. *)
val parse :
  ?argc:int -> ?argv:string array -> ?auto_help:bool ->
  ((char * string * string) * (unit -> unit) option * opt_spec) list ->
  (string -> unit) -> string -> unit
(** The callback functions provided in [longopts] and by [process] are called
  * in the order of the options/arguments on the command line.
  * This is in contrast with the default behaviour of GNU getopt, where all
  * the plain arguments are first shifted to the end, and then the [process]
  * calls are made.  This is easy to emulate by using [process] to just fill a
  * list that will be processed after; the current behaviour is that of
  * GNU getopt when a ['-'] is appended to [optstring].
  * @param argc defaults to [Array.length Sys.argv]
  * @param argv defaults to [Sys.argc]
  * @param auto_help whether a -h/--help option must be generated if missing *)
